@@ -11,8 +11,8 @@ Examples:
     Load all the necessary data using a :func:`pyrost.STData.load` function.
 
     >>> import pyrost as rst
-    >>> files = rst.CXIStore(input_files='data.cxi', output_file='data.cxi')
-    >>> data = rst.STData(files)
+    >>> inp_file = rst.CXIStore('data.cxi')
+    >>> data = rst.STData(input_file=inp_file)
     >>> data = data.load()
 """
 from __future__ import annotations
@@ -73,15 +73,9 @@ class Transform():
     def forward(self, inp: np.ndarray) -> np.ndarray:
         raise NotImplementedError
 
-    def forward_points(self, pts: np.ndarray) -> np.ndarray:
-        raise NotImplementedError
-
     def backward(self, inp: np.ndarray, out: Optional[np.ndarray]=None) -> np.ndarray:
         raise NotImplementedError
 
-    def backward_points(self, pts: np.ndarray) -> np.ndarray:
-        raise NotImplementedError
-
     def integrate(self, axis: int) -> Transform:
         """Return a transform version for the dataset integrated
         along the axis.
@@ -110,15 +104,15 @@ class Crop(Transform):
     """Crop transform. Crops a frame according to a region of interest.
 
     Attributes:
-        roi : Region of interest. Comprised of four elements `[[y_min, x_min],
-            [y_max, x_max]]`.
+        roi : Region of interest. Comprised of four elements `[y_min, y_max,
+            x_min, x_max]`.
         shape : Data frame shape.
     """
     def __init__(self, roi: Iterable[int], shape: Optional[Tuple[int, int]]=None) -> None:
         """
         Args:
-            roi : Region of interest. Comprised of four elements `[[y_min, x_min],
-                [y_max, x_max]]`.
+            roi : Region of interest. Comprised of four elements `[y_min, y_max,
+                x_min, x_max]`.
             shape : Data frame shape.
         """
         super().__init__(shape=shape)
@@ -134,21 +128,10 @@ class Crop(Transform):
             Output data array.
         """
         if self.check_shape(inp.shape[-2:]):
-            return inp[..., self.roi[0][0]:self.roi[1][0], self.roi[0][1]:self.roi[1][1]]
+            return inp[..., self.roi[0]:self.roi[1], self.roi[2]:self.roi[3]]
 
         raise ValueError(f'input array has invalid shape: {str(inp.shape):s}')
 
-    def forward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Apply the transform to a set of points.
-
-        Args:
-            pts : Input array of points.
-
-        Returns:
-            Output array of points.
-        """
-        return pts - self.roi[0]
-
     def backward(self, inp: np.ndarray, out: Optional[np.ndarray]=None) -> np.ndarray:
         """Tranform back a data array.
 
@@ -163,22 +146,11 @@ class Crop(Transform):
             out = np.zeros(inp.shape[:-2] + self.shape, dtype=inp.dtype)
 
         if self.check_shape(out.shape[-2:]):
-            out[..., self.roi[0][0]:self.roi[1][0], self.roi[0][1]:self.roi[1][1]] = inp
+            out[..., self.roi[0]:self.roi[1], self.roi[2]:self.roi[3]] = inp
             return out
 
         raise ValueError(f'output array has invalid shape: {str(out.shape):s}')
 
-    def backward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Tranform back an array of points.
-
-        Args:
-            pts : Input tranformed array of points.
-
-        Returns:
-            Output array of points.
-        """
-        return pts + self.roi[0]
-
     def integrate(self, axis: int) -> Crop:
         """Return a transform version for the dataset integrated
         along the axis.
@@ -190,8 +162,12 @@ class Crop(Transform):
             A new transform version.
         """
         pdict = self.state_dict()
-        pdict['roi'][0][axis] = 0
-        pdict['roi'][1][axis] = 1
+        if axis == 0:
+            pdict['roi'] = (0, 1, pdict['roi'][2], pdict['roi'][3])
+        elif axis == 1:
+            pdict['roi'] = (pdict['roi'][0], pdict['roi'][1], 0, 1)
+        else:
+            raise ValueError('Axis must be equal to 0 or 1')
 
         if pdict['shape']:
             if axis == 0:
@@ -241,17 +217,6 @@ class Downscale(Transform):
 
         raise ValueError(f'input array has invalid shape: {str(inp.shape):s}')
 
-    def forward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Apply the transform to a set of points.
-
-        Args:
-            pts : Input array of points.
-
-        Returns:
-            Output array of points.
-        """
-        return pts / self.scale
-
     def backward(self, inp: np.ndarray, out: Optional[np.ndarray]=None) -> np.ndarray:
         """Tranform back a data array.
 
@@ -272,17 +237,6 @@ class Downscale(Transform):
 
         raise ValueError(f'output array has invalid shape: {str(out.shape):s}')
 
-    def backward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Tranform back an array of points.
-
-        Args:
-            pts : Input tranformed array of points.
-
-        Returns:
-            Output array of points.
-        """
-        return pts * self.scale
-
     def state_dict(self) -> Dict[str, Any]:
         """Returns the state of the transform as a dict.
 
@@ -324,18 +278,6 @@ class Mirror(Transform):
 
         raise ValueError(f'input array has invalid shape: {str(inp.shape):s}')
 
-    def forward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Apply the transform to a set of points.
-
-        Args:
-            pts : Input array of points.
-
-        Returns:
-            Output array of points.
-        """
-        pts[:, self.axis] = self.shape[self.axis] - pts[:, self.axis]
-        return pts
-
     def backward(self, inp: np.ndarray, out: Optional[np.ndarray]=None) -> np.ndarray:
         """Tranform back a data array.
 
@@ -355,17 +297,6 @@ class Mirror(Transform):
 
         raise ValueError(f'output array has invalid shape: {str(out.shape):s}')
 
-    def backward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Tranform back an array of points.
-
-        Args:
-            pts : Input tranformed array of points.
-
-        Returns:
-            Output array of points.
-        """
-        return self.forward_points(pts)
-
     def state_dict(self) -> Dict[str, Any]:
         """Returns the state of the transform as a dict.
 
@@ -419,19 +350,6 @@ class ComposeTransforms(Transform):
             inp = transform.forward(inp)
         return inp
 
-    def forward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Apply the transform to a set of points.
-
-        Args:
-            pts : Input array of points.
-
-        Returns:
-            Output array of points.
-        """
-        for transform in self:
-            pts = transform.forward_points(pts)
-        return pts
-
     def backward(self, inp: np.ndarray, out: Optional[np.ndarray]=None) -> np.ndarray:
         """Tranform back a data array.
 
@@ -446,19 +364,6 @@ class ComposeTransforms(Transform):
             inp = transform.backward(inp)
         return self[0].backward(inp, out)
 
-    def backward_points(self, pts: np.ndarray) -> np.ndarray:
-        """Tranform back an array of points.
-
-        Args:
-            pts : Input tranformed array of points.
-
-        Returns:
-            Output array of points.
-        """
-        for transform in self[::-1]:
-            pts = transform.backward_points(pts)
-        return pts
-
     def integrate(self, axis: int) -> ComposeTransforms:
         """Return a transform version for the dataset integrated
         along the axis.
@@ -498,7 +403,7 @@ class STData(DataContainer):
     be tranformed using any of the :class:`pyrost.Transform` classes.
 
     Attributes:
-        files : HDF5 or CXI file handler.
+        input_file : HDF5 or CXI file handler.
         transform : Frames transform object.
 
     Notes:
@@ -520,6 +425,7 @@ class STData(DataContainer):
         * good_frames : An array of good frames' indices.
         * mask : Bad pixels mask.
         * num_threads : Number of threads used in computations.
+        * output_file : Output file handler.
         * phase : Phase profile of lens' aberrations.
         * pixel_aberrations : Lens' aberrations along the horizontal and
           vertical axes in pixels.
@@ -531,15 +437,15 @@ class STData(DataContainer):
         * whitefields : Set of dynamic white-fields for each of the measured
           images.
     """
-    attr_set = {'files'}
+    attr_set = {'input_file'}
     init_set = {'basis_vectors', 'data', 'distance', 'frames', 'translations', 'wavelength',
                 'x_pixel_size', 'y_pixel_size', 'defocus_x', 'defocus_y', 'good_frames',
-                'mask', 'num_threads', 'phase', 'pixel_aberrations',  'pixel_translations',
-                'reference_image', 'scale_map', 'transform', 'whitefield', 'whitefields'}
-    is_points = {}
+                'mask', 'num_threads', 'output_file', 'phase', 'pixel_aberrations',
+                'pixel_translations', 'reference_image', 'scale_map', 'transform', 'whitefield',
+                'whitefields'}
 
     # Necessary attributes
-    files               : CXIStore
+    input_file         : CXIStore
     transform           : Transform
 
     # Optional attributes
@@ -549,6 +455,7 @@ class STData(DataContainer):
     defocus_y           : Optional[float]
     distance            : Optional[np.ndarray]
     frames              : Optional[np.ndarray]
+    output_file         : Optional[CXIStore]
     phase               : Optional[np.ndarray]
     pixel_aberrations   : Optional[np.ndarray]
     reference_image     : Optional[np.ndarray]
@@ -566,11 +473,12 @@ class STData(DataContainer):
     pixel_translations  : Optional[np.ndarray]
     whitefield          : Optional[np.ndarray]
 
-    def __init__(self, files: CXIStore, transform: Optional[Transform]=None,
-                 **kwargs: Union[int, float, np.ndarray]) -> None:
+    def __init__(self, input_file: CXIStore, output_file: Optional[CXIStore]=None,
+                 transform: Optional[Transform]=None, **kwargs: Union[int, float, np.ndarray]) -> None:
         """
         Args:
-            files : HDF5 or CXI file handler.
+            input_file : HDF5 or CXI file handler of input files.
+            ouput_file : Output file handler.
             transform : Frames transform object.
             kwargs : Dictionary of the necessary and optional data attributes specified
                 in :class:`pyrost.STData` notes. All the necessary attributes must be
@@ -580,13 +488,18 @@ class STData(DataContainer):
             ValueError : If any of the necessary attributes specified in :class:`pyrost.STData`
                 notes have not been provided.
         """
-        super(STData, self).__init__(files=files, transform=transform, **kwargs)
+        super(STData, self).__init__(input_file=input_file, output_file=output_file, 
+                                     transform=transform, **kwargs)
 
         self._init_functions(num_threads=lambda: np.clip(1, 64, cpu_count()))
+        if self.shape[0] > 0:
+            self._init_functions(good_frames=lambda: np.arange(self.shape[0]))
         if self._isdata:
-            self._init_functions(good_frames=lambda: np.arange(self.data.shape[0]),
-                                 mask=lambda: np.ones(self.data.shape, dtype=bool),
-                                 whitefield=self._whitefield)
+            self._init_functions(mask=lambda: np.ones(self.shape, dtype=bool))
+            func = lambda: median(data=self.data[self.good_frames], axis=0,
+                                  mask=self.mask[self.good_frames],
+                                  num_threads=self.num_threads)
+            self._init_functions(whitefield=func)
         if self._isdefocus:
             self._init_functions(defocus_y=lambda: self.get('defocus_x', None),
                                  pixel_translations=self._pixel_translations)
@@ -605,6 +518,20 @@ class STData(DataContainer):
     def _isphase(self) -> bool:
         return not self.pixel_aberrations is None and not self.phase is None
 
+    @property
+    def shape(self) -> Tuple[int, int, int]:
+        shape = [0, 0, 0]
+        for attr, data in self.items():
+            if attr in self.input_file.protocol and data is not None:
+                kind = self.input_file.protocol.get_kind(attr)
+                if kind == 'sequence':
+                    shape[0] = data.shape[0]
+                if kind == 'stack':
+                    shape[:] = data.shape
+                if kind == 'frame':
+                    shape[1:] = data.shape
+        return tuple(shape)
+
     def _basis_vectors(self) -> np.ndarray:
         def get_axis(transform: Transform):
             return transform.state_dict().get('axis', -1)
@@ -632,14 +559,9 @@ class STData(DataContainer):
         pixel_translations -= pixel_translations.mean(axis=0)
         return pixel_translations
 
-    def _whitefield(self) -> np.ndarray:
-        return median(data=self.data[self.good_frames], axis=0,
-                      mask=self.mask[self.good_frames],
-                      num_threads=self.num_threads)
-
     def _transform_attribute(self, attr: str, data: np.ndarray, transform: Transform,
                              mode: str='forward') -> np.ndarray:
-        kind = self.files.protocol.get_kind(attr)
+        kind = self.input_file.protocol.get_kind(attr)
         if kind in ['stack', 'frame']:
             if mode == 'forward':
                 data = transform.forward(data)
@@ -647,11 +569,6 @@ class STData(DataContainer):
                 data = transform.backward(data)
             else:
                 raise ValueError(f'Invalid mode keyword: {mode}')
-        if attr in self.is_points:
-            if data.shape[-1] != 2:
-                raise ValueError(f"'{attr}' has invalid shape: {str(data.shape)}")
-
-            data = self.transform.forward_points(data)
 
         return data
 
@@ -664,8 +581,8 @@ class STData(DataContainer):
         Returns:
             Pixel mapping array.
         """
-        if self._isdata:
-            pixel_map = np.indices(self.data.shape[1:], dtype=dtype)
+        if sum(self.shape[1:]):
+            pixel_map = np.indices(self.shape[1:], dtype=dtype)
 
             if self._isdefocus:
                 if self.defocus_y < 0.0 and pixel_map[0, 0, 0] < pixel_map[0, -1, 0]:
@@ -677,14 +594,14 @@ class STData(DataContainer):
         raise AttributeError('Data has not been loaded')
 
     @dict_to_object
-    def load(self, attributes: Union[str, List[str], None]=None, indices: Iterable[int]=None,
+    def load(self, attributes: Union[str, List[str], None]=None, idxs: Optional[Iterable[int]]=None,
              processes: int=1, verbose: bool=True) -> STData:
-        """Load data attributes from the input files in `files` file handler object.
+        """Load data attributes from the input files in `input_file` file handler object.
 
         Args:
             attributes : List of attributes to load. Loads all the data attributes
                 contained in the file(s) by default.
-            indices : List of frame indices to load.
+            idxs : List of frame indices to load.
             processes : Number of parallel workers used during the loading.
             verbose : Set the verbosity of the loading process.
 
@@ -695,26 +612,26 @@ class STData(DataContainer):
         Returns:
             New :class:`STData` object with the attributes loaded.
         """
-        with self.files:
-            self.files.update_indices()
+        with self.input_file:
+            self.input_file.update_indices()
 
             if attributes is None:
-                attributes = [attr for attr in self.files.keys()
+                attributes = [attr for attr in self.input_file.keys()
                               if attr in self.init_set]
             else:
-                attributes = self.files.protocol.str_to_list(attributes)
+                attributes = self.input_file.protocol.str_to_list(attributes)
 
-            if indices is None:
-                indices = self.files.indices()
-            data_dict = {'frames': indices}
+            if idxs is None:
+                idxs = self.input_file.indices()
+            data_dict = {'frames': idxs}
 
             for attr in attributes:
-                if attr not in self.files.keys():
+                if attr not in self.input_file.keys():
                     raise ValueError(f"No '{attr}' attribute in the input files")
                 if attr not in self.init_set:
                     raise ValueError(f"Invalid attribute: '{attr}'")
 
-                data = self.files.load_attribute(attr, indices, processes, verbose)
+                data = self.input_file.load_attribute(attr, idxs, processes, verbose)
 
                 if self.transform and data is not None:
                     data = self._transform_attribute(attr, data, self.transform)
@@ -723,7 +640,7 @@ class STData(DataContainer):
 
         return data_dict
 
-    def save(self, attributes: Union[str, List[str], None]=None, apply_transform=False,
+    def save(self, attributes: Union[str, List[str], None]=None, apply_transform: bool=False,
              mode: str='append', idxs: Optional[Iterable[int]]=None) -> None:
         """Save data arrays of the data attributes contained in the container to
         an output file.
@@ -731,22 +648,33 @@ class STData(DataContainer):
         Args:
             attributes : List of attributes to save. Saves all the data attributes
                 contained in the container by default.
-            apply_transform : Apply `transform` to the data arrays if True.
+            apply_transform : Apply `transform` to the data arrays if True. The
+                saved data will be expanded to comply with the original shape of
+                detector grid.
             mode : Writing mode:
 
                 * `append` : Append the data array to already existing dataset.
                 * `insert` : Insert the data under the given indices `idxs`.
                 * `overwrite` : Overwrite the existing dataset.
 
+            idxs : A set of frame indices where the data is saved if `mode` is
+                `insert`.
+
             verbose : Set the verbosity of the loading process.
+
+        Raises:
+            ValueError : If `output_file` is not defined inside the container.
         """
+        if self.output_file is None:
+            raise ValueError("'output_file' is not defined inside the container")
+
         if attributes is None:
             attributes = list(self.contents())
-        with self.files:
-            for attr in self.files.protocol.str_to_list(attributes):
+        with self.output_file:
+            for attr in self.output_file.protocol.str_to_list(attributes):
                 data = self.get(attr)
-                if attr in self.files.protocol and data is not None:
-                    kind = self.files.protocol.get_kind(attr)
+                if attr in self.output_file.protocol and data is not None:
+                    kind = self.output_file.protocol.get_kind(attr)
 
                     if kind in ['stack', 'sequence']:
                         data = data[self.good_frames]
@@ -755,7 +683,7 @@ class STData(DataContainer):
                         data = self._transform_attribute(attr, data, self.transform,
                                                          mode='backward')
 
-                    self.files.save_attribute(attr, np.asarray(data), mode=mode, idxs=idxs)
+                    self.output_file.save_attribute(attr, np.asarray(data), mode=mode, idxs=idxs)
 
     @dict_to_object
     def clear(self, attributes: Union[str, List[str], None]=None) -> STData:
@@ -770,16 +698,31 @@ class STData(DataContainer):
         if attributes is None:
             attributes = self.keys()
         data_dict = {}
-        for attr in attributes:
+        for attr in self.input_file.protocol.str_to_list(attributes):
             data = self.get(attr)
-            if attr in self.files.protocol and data is not None:
+            if attr in self and isinstance(data, np.ndarray):
                 data_dict[attr] = None
         return data_dict
 
+    @dict_to_object
+    def update_output_file(self, output_file: CXIStore) -> STData:
+        """Return a new :class:`STData` object with the new output
+        file handler.
+
+        Args:
+            output_file : A new output file handler.
+
+        Returns:
+            New :class:`STData` object with the new output file
+            handler.
+        """
+        return {'output_file': output_file}
+
     @dict_to_object
     def integrate_data(self, axis: int=0) -> STData:
         """Return a new :class:`STData` object with the `data` summed
-        over the `axis`.
+        over the `axis`. Clear all the 2D and 3D data attributes inside the
+        container.
 
         Args:
             axis : Axis along which a sum is performed.
@@ -795,14 +738,12 @@ class STData(DataContainer):
                 data_dict['transform'] = self.transform.integrate(axis)
 
             for attr, data in self.items():
-                if attr in self.files.protocol and data is not None:
-                    kind = self.files.protocol.get_kind(attr)
+                if attr in self.input_file.protocol and data is not None:
+                    kind = self.input_file.protocol.get_kind(attr)
                     if kind in ['stack', 'frame']:
                         data_dict[attr] = None
 
-            data = np.zeros(self.data.shape, self.data.dtype)
-            data[self.good_frames] = self.data * self.mask
-            data_dict['data'] = data.sum(axis=axis - 2, keepdims=True)
+            data_dict['data'] = (self.data * self.mask).sum(axis=axis - 2, keepdims=True)
 
             return data_dict
 
@@ -810,16 +751,16 @@ class STData(DataContainer):
 
     @dict_to_object
     def mask_frames(self, good_frames: Optional[Iterable[int]]=None) -> STData:
-        """Return a new :class:`STData` object with the updated
-        good frames mask. Mask empty frames by default.
+        """Return a new :class:`STData` object with the updated good frames
+        mask. Mask empty frames by default.
 
         Args:
-            good_frames : List of good frames' indices. Masks empty
-                frames if not provided.
+            good_frames : List of good frames' indices. Masks empty frames
+                if not provided.
 
         Returns:
-            New :class:`STData` object with the updated `good_frames`
-            and `whitefield`.
+            New :class:`STData` object with the updated `good_frames` and
+            `whitefield`.
         """
         if good_frames is None:
             good_frames = np.where(self.data.sum(axis=(1, 2)) > 0)[0]
@@ -855,18 +796,19 @@ class STData(DataContainer):
         elif update == 'multiply':
             data = self.data * self.mask
         else:
-            raise ValueError(f'Invalid update keyword: {update}')
+            raise ValueError(f'Invalid update keyword: {update:s}')
 
         if method == 'no-bad':
-            mask = np.ones(data.shape, dtype=bool)
+            mask = np.ones(self.shape, dtype=bool)
         elif method == 'range-bad':
             mask = (data >= vmin) & (data < vmax)
         elif method == 'perc-bad':
-            offsets = (data - np.median(data))
+            average = median_filter(data, (1, 3, 3), num_threads=self.num_threads)
+            offsets = (data.astype(np.int32) - average.astype(np.int32))
             mask = (offsets >= np.percentile(offsets, pmin)) & \
                    (offsets <= np.percentile(offsets, pmax))
         else:
-            ValueError(f'Invalid method argument: {method}')
+            ValueError('invalid method argument')
 
         if update == 'reset':
             return {'mask': mask, 'whitefield': None}
@@ -885,9 +827,18 @@ class STData(DataContainer):
             New :class:`STData` object with the updated transform object.
         """
         data_dict = {'transform': transform}
+
+        if self.transform is None:
+            for attr, data in self.items():
+                if attr in self.input_file.protocol and data is not None:
+                    data_dict[attr] = self._transform_attribute(attr, data, transform)
+            return data_dict
+
         for attr, data in self.items():
-            if attr in self.files.protocol and data is not None:
-                data_dict[attr] = self._transform_attribute(attr, data, transform)
+            if attr in self.input_file.protocol and data is not None:
+                kind = self.input_file.protocol.get_kind(attr)
+                if kind in ['stack', 'frame']:
+                    data_dict[attr] = None
         return data_dict
 
     @dict_to_object
@@ -1013,15 +964,16 @@ class STData(DataContainer):
                       return_extra: bool=False, verbose: bool=True) -> Tuple[List[float], Dict[str, np.ndarray]]:
         r"""Calculate a set of reference images for each defocus in `defoci` and
         return an average R-characteristic of an image (the higher the value the
-        sharper reference image is). Return the intermediate results if `return_extra`
+        sharper reference image is). The kernel bandwidth `hval` is automatically
+        estimated by default. Return the intermediate results if `return_extra`
         is True.
 
         Args:
             defoci_x : Array of defocus distances along the horizontal detector axis [m].
             defoci_y : Array of defocus distances along the vertical detector axis [m].
             hval : Kernel bandwidth in pixels for the reference image update. Estimated
-                with :func:`pyrost.SpeckleTracking.find_hopt` at each defocus distance if
-                None.
+                with :func:`pyrost.SpeckleTracking.find_hopt` for an average defocus value
+                if None.
             size : Local variance filter size in pixels.
             extra_args : Extra arguments parser to the :func:`STData.get_st` and
                 :func:`SpeckleTracking.update_reference` methods. The following
@@ -1089,13 +1041,12 @@ class STData(DataContainer):
         r_vals = []
         extra = {'reference_image': [], 'r_image': []}
         kernel = np.ones(int(size)) / size
-        df0_x, df0_y = defoci_x.ravel()[0], defoci_y.ravel()[0]
+        df0_x, df0_y = defoci_x.mean(), defoci_y.mean()
         st_obj = self.update_defocus(df0_x, df0_y).get_st(ds_y=ds_y, ds_x=ds_x,
                                                           aberrations=aberrations,
                                                           ff_correction=ff_correction)
-        update_hval = hval is None
-        if update_hval:
-            hval = 1.0
+        if hval is None:
+            hval = st_obj.find_hopt(method=ref_method)
 
         for df1_x, df1_y in tqdm(zip(defoci_x.ravel(), defoci_y.ravel()),
                                total=len(defoci_x), disable=not verbose,
@@ -1103,8 +1054,6 @@ class STData(DataContainer):
             st_obj.di_pix *= np.abs(df0_y / df1_y)
             st_obj.dj_pix *= np.abs(df0_x / df1_x)
             df0_x, df0_y = df1_x, df1_y
-            if update_hval:
-                hval = st_obj.find_hopt(hval, method=ref_method)
             st_obj.update_reference.inplace_update(hval=hval, method=ref_method)
             extra['reference_image'].append(st_obj.reference_image)
             mean = st_obj.reference_image.copy()
@@ -1210,11 +1159,11 @@ class STData(DataContainer):
             raise ValueError(f'invalid axis value: {axis:d}')
 
         data_dict['defocus'] = np.abs(data_dict['defocus'])
-        if center <= self.data.shape[axis - 2]:
-            data_dict['pixels'] = np.arange(self.data.shape[axis - 2]) - center
+        if center <= self.shape[axis - 2]:
+            data_dict['pixels'] = np.arange(self.shape[axis - 2]) - center
             data_dict['pixel_aberrations'] = data_dict['pixel_aberrations'][axis].mean(axis=1 - axis)
-        elif center >= self.data.shape[axis - 2] - 1:
-            data_dict['pixels'] = center - np.arange(self.data.shape[axis - 2])
+        elif center >= self.shape[axis - 2] - 1:
+            data_dict['pixels'] = center - np.arange(self.shape[axis - 2])
             idxs = np.argsort(data_dict['pixels'])
             data_dict['pixel_aberrations'] = -data_dict['pixel_aberrations'][axis].mean(axis=1 - axis)[idxs]
             data_dict['pixels'] = data_dict['pixels'][idxs]
@@ -1242,21 +1191,25 @@ class STData(DataContainer):
                     normalization using eigen flat fields in X-ray imaging," Opt.
                     Express 23, 27975-27989 (2015).
         """
-        dtype = np.promote_types(self.whitefield.dtype, int)
-        cor_data = np.zeros(self.data.shape, dtype=dtype)
-        np.subtract(self.data, self.whitefield, dtype=dtype,
-                    where=self.mask, out=cor_data)
-        mat_svd = np.tensordot(cor_data, cor_data, axes=((1, 2), (1, 2)))
-        eig_vals, eig_vecs = np.linalg.eig(mat_svd)
-        effs = np.tensordot(eig_vecs, cor_data, axes=((0,), (0,)))
-        return cor_data, effs, eig_vals / eig_vals.sum()
+        if self._isdata:
+
+            dtype = np.promote_types(self.whitefield.dtype, int)
+            cor_data = np.zeros(self.shape, dtype=dtype)
+            np.subtract(self.data, self.whitefield, dtype=dtype,
+                        where=self.mask, out=cor_data)
+            mat_svd = np.tensordot(cor_data, cor_data, axes=((1, 2), (1, 2)))
+            eig_vals, eig_vecs = np.linalg.eig(mat_svd)
+            effs = np.tensordot(eig_vecs, cor_data, axes=((0,), (0,)))
+            return cor_data, effs, eig_vals / eig_vals.sum()
+
+        raise AttributeError('Data has not been loaded')
 
     @dict_to_object
     def update_whitefields(self, method: str='median', size: int=11,
                            cor_data: Optional[np.ndarray]=None,
                            effs: Optional[np.ndarray]=None) -> STData:
         """Return a new :class:`STData` object with a new set of dynamic whitefields.
-        The flatfields are generated by the dint of median filtering or Principal
+        A set of whitefields are generated by the dint of median filtering or Principal
         Component Analysis [PCA]_.
 
         Args:
@@ -1286,23 +1239,29 @@ class STData(DataContainer):
         See Also:
             :func:`pyrost.STData.get_pca` : Method to generate eigen flatfields.
         """
-        if method == 'median':
-            outliers = np.abs(self.data - self.whitefield) < 3 * np.sqrt(self.whitefield)
-            whitefields = median_filter(self.data, size=(size, 1, 1), mask=outliers,
-                                        num_threads=self.num_threads)
-        elif method == 'pca':
-            if cor_data is None:
-                dtype = np.promote_types(self.whitefield.dtype, int)
-                cor_data = np.zeros(self.data.shape, dtype=dtype)
-                np.subtract(self.data, self.whitefield, dtype=dtype,
-                            where=self.mask, out=cor_data)
-            if effs is None:
-                raise ValueError('No eigen flat fields were provided')
+        if self._isdata:
 
-            weights = np.tensordot(cor_data, effs, axes=((1, 2), (1, 2))) / \
-                      np.sum(effs * effs, axis=(1, 2))
-            whitefields = np.tensordot(weights, effs, axes=((1,), (0,))) + self.whitefield
-        else:
-            raise ValueError('Invalid method argument')
+            if method == 'median':
+                offsets = np.abs(self.data - self.whitefield)
+                outliers = offsets < 3 * np.sqrt(self.whitefield)
+                whitefields = median_filter(self.data, size=(size, 1, 1), mask=outliers,
+                                            num_threads=self.num_threads)
+            elif method == 'pca':
+                if cor_data is None:
+                    dtype = np.promote_types(self.whitefield.dtype, int)
+                    cor_data = np.zeros(self.shape, dtype=dtype)
+                    np.subtract(self.data, self.whitefield, dtype=dtype,
+                                where=self.mask, out=cor_data)
+                if effs is None:
+                    raise ValueError('No eigen flat fields were provided')
 
-        return {'whitefields': whitefields}
+                weights = np.tensordot(cor_data, effs, axes=((1, 2), (1, 2)))
+                weights /= np.sum(effs * effs, axis=(1, 2))
+                whitefields = np.tensordot(weights, effs, axes=((1,), (0,)))
+                whitefields += self.whitefield
+            else:
+                raise ValueError('Invalid method argument')
+
+            return {'whitefields': whitefields}
+
+        raise ValueError('Data has not been loaded')
